package net.sergeych.crypto2

import com.ionspin.kotlin.crypto.util.encodeToUByteArray
import net.sergeych.bipack.BipackEncoder

/**
 * Some key able to encrypt data with optional random fill that conceals message size
 * when needed.
 *
 * It is not serializable by design.
 * Custom implementations are, see [SymmetricKey] for example.
 */
interface EncryptingKey {
    /**
     * Authenticated encrypting with optional random fill to protect from message size analysis.
     * Note that [randomFill] if present should be positive.
     */
    fun encrypt(plainData: UByteArray,randomFill: IntRange?=null): UByteArray

    fun encrypt(plainText: String,randomFill: IntRange? = null): UByteArray =
        encrypt(plainText.encodeToUByteArray(),randomFill)
}

inline fun <reified T>EncryptingKey.encryptObject(value: T,randomFill: IntRange? = null): UByteArray =
    encrypt(BipackEncoder.encode(value).toUByteArray(),randomFill)
